public class Turtle {
	/**
	* The maximum level of turtle tiredness. 
	* When reached this level, the turtle cannot move and need sleeping.
	*/
	public final int TIRED=5;

    private final int ISLAND_LENGTH;
	private final int ISLAND_WIDTH;


	private Location home;

	private Location curLocation;
	private int tiredness=0;

	/**
	* Constructs a turtle, given island dimensions and home
	* location. The turtle initial coordinates are (1,1) and level of tiredness is zero.
	* @param islandLength - number of horizontal cells
	* @param islandWidth - number of vertical cells
	* @param home - home <code>Location</code>
	*/
	public Turtle(int islandLength, int islandWidth, Location home) {
		this.ISLAND_LENGTH = islandLength;
    	this.ISLAND_WIDTH = islandWidth;
		this.home = new Location(home);
		this.curLocation= new Location(1,1);
	}


	/**
	* Returns turtle home location.
	*/
 	public Location getHome() {
 		return new Location(home);
 	}

	/**
	* Returns turtle level of tiredness.
	*/
 	public int getLevelOfTired() {
 		return this.tiredness;
 	}

	/**
	* Returns turtle current location.
	*/
 	public Location getLocation() {
 		return new Location(this.curLocation);
 	}

	/**
	* Causes the turtle to move back to the home location, regardless of tiredness level.
	* When the turtle returns home, it AUTOMATICALLY goes to sleep.
	*/
 	public void goHome() {
 		(this.curLocation).setRow((this.home).getRow());
    	(this.curLocation).setCol((this.home).getCol());
		this.sleep();
 	}


	/**
	* Causes the turtle to move one point to the east (right).
	* The move increases level of tiredness by one.
	* In case the turtle cannot move right because the island is over,
	* or the tutle's level of tiredness is at maximum level,
	* it will not move, and nothing will happen.
	*/
 	public void goEast() {
 		if ((this.curLocation).getCol() < this.ISLAND_LENGTH && 
			this.tiredness < this.TIRED) {
			this.curLocation.addToCol(1);
 			this.tiredness++;
		}
 	}

	/**
	* Causes the turtle to move one point to the the west(left).
	* The move increases level of tiredness by one.
	* In case the turtle cannot move left because the island is over,
	* or the tutle's level of tiredness is at maximum level,
	* it will not move, and nothing will happen.
	*/
 	public void goWest() {
		if ((this.curLocation).getCol()>1 && this.tiredness<TIRED) {
			this.curLocation.addToCol(-1);
 			this.tiredness++;
		}
 	}

	/**
	* Causes the turtle to move one point to the north (up).
	* The move increases level of tiredness by one.
	* In case the turtle cannot move up because the island is over,
	* or the tutle's level of tiredness is at maximum level,
	* it will not move, and nothing will happen.
	*/
 	public void goNorth() {
		if ((this.curLocation).getRow()>1 && this.tiredness<TIRED) {
			this.curLocation.addToRow(-1);
 			this.tiredness++;
		}
	}
   
	/**
	* Causes the turtle to move one point to the south (down).
	* The move increases level of tiredness by one.
	* In case the turtle cannot move south because the island is over,
	* or the tutle's level of tiredness is at maximum level,
	* it will not move, and nothing will happen.
	*/
	public void goSouth() {
		if ((this.curLocation).getRow()<ISLAND_WIDTH && 
			this.tiredness<TIRED) {
			this.curLocation.addToRow(1);
 			this.tiredness++;
		}
 	}

	/**
	* Causes the turtle to go to sleep, restoring level of tiredness to zero.
	*/
 	public void sleep() {
		this.tiredness=0;
 	}

}
