/*
 * TurtleIsland.java
 *
 */

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

public class TurtleIsland extends JFrame implements ActionListener {
	public static final int ROWS=8;
	public static final int COLS=8;

	private static final int ISLAND_WIDTH=780;
	private static final int ISLAND_HEIGHT=427;

	private static final int CELL_WIDTH=ISLAND_WIDTH/COLS;
	private static final int CELL_HEIGHT=ISLAND_HEIGHT/ROWS;

	private static final int ANIM_HMOVE=2; 
	private static final int ANIM_VMOVE=2;

	private static final int HCOUNT=CELL_WIDTH/ANIM_HMOVE;
	private static final int VCOUNT=CELL_HEIGHT/ANIM_VMOVE;

	private static final double ROUND_OFF_V=(double)ISLAND_HEIGHT/ROWS - VCOUNT*ANIM_VMOVE;
	private static final double ROUND_OFF_H=(double)ISLAND_WIDTH/COLS - HCOUNT*ANIM_HMOVE;
	private double round_off_h=0;
	private double round_off_v=0;


	private Controls controlPanel;
	
	private static final int HOME_X=6;
	private static final int HOME_Y=3;
	private Turtle turtle=new Turtle(ROWS,COLS, new Location(HOME_Y,HOME_X));
	
	
	private static final int JWIDTH=86;
	private static final int JHEIGHT=36;
	private static final int JX1=(CELL_WIDTH-JWIDTH)/2;
	private static final int JY1=(CELL_HEIGHT-JHEIGHT)/2;
	private static final int JHOME_X=JX1 + CELL_WIDTH*(HOME_X-1);
	private static final int JHOME_Y=JY1 + CELL_HEIGHT*(HOME_Y-1);
	private JTurtle jturtle;
	
	private javax.swing.Timer animator;

	int count=0;
	int x,y;
	int hMove,vMove;
	int currentRow=1, currentColumn=1;
	int tired=0;

	public TurtleIsland() {
		super("Turtle Island");
		//System.out.println("JHOME_X: "+JHOME_X+" JHOME_Y: "+JHOME_Y);
		
		getContentPane().setLayout(new BorderLayout());

		JLayeredPane jLayeredPane = new JLayeredPane();
		jLayeredPane.setBorder(new EtchedBorder());
		getContentPane().add(jLayeredPane, BorderLayout.CENTER);


		try {
			ImageIcon icon=new ImageIcon(ClassLoader.getSystemResource("pic/island.jpg"));
			JLabel island=new JLabel(icon);
			jLayeredPane.add(island, JLayeredPane.DEFAULT_LAYER);
			island.setBounds(0,0,ISLAND_WIDTH,ISLAND_HEIGHT);
		} catch(Exception e) {
			System.out.println("The 'pic' directory is not found! Please refer to exercise description.");
			System.exit(1);
		}

		jturtle=new JTurtle(true);
		jLayeredPane.add(jturtle, JLayeredPane.DRAG_LAYER);
		jturtle.setSize(JWIDTH, JHEIGHT);
		jturtle.setLocation(JX1, JY1);

		GridPanel gridPanel=new GridPanel(ROWS,COLS);
		jLayeredPane.add(gridPanel, JLayeredPane.PALETTE_LAYER);
		//gridPanel.setSize(200, 100);
		gridPanel.setLocation(0, 0);


		controlPanel=new Controls(this);
		getContentPane().add(controlPanel, BorderLayout.SOUTH);
		controlPanel.showLocation(currentRow,currentColumn);
		controlPanel.showTiredness(tired);


		//setDefaultCloseOperation(EXIT_ON_CLOSE);

		setResizable (false);
		addWindowListener (new java.awt.event.WindowAdapter () {
			public void windowClosing (java.awt.event.WindowEvent evt) {
				System.exit(0);
			}
		}
		);

		pack();
		setSize(780,557);
		validate();

		Dimension d=jLayeredPane.getSize();
		gridPanel.setSize(d.width, d.height);
		animator = new javax.swing.Timer(66, this);
	}

	public void moveLeft() {
		if (count!=0) return;
		turtle.goWest();
		if (currentColumn==1 || tired==turtle.TIRED) {
			checkTurtle();
			return;
		}

		controlPanel.disableButtons();
		
		currentColumn-=1;
		tired++;
		checkTurtle();
		
		jturtle.runLeft();
		hMove=-ANIM_HMOVE;
		vMove=0;
		count=HCOUNT;

		round_off_h-=ROUND_OFF_H;
		if (-round_off_h>=ANIM_HMOVE) {
			count+=1;
			round_off_h+=ANIM_HMOVE;
		}
		animator.start();
	}

	public void moveRight() {
		if (count!=0) return;
		turtle.goEast();
		if (tired==turtle.TIRED || currentColumn==COLS) {
			checkTurtle();
			return;
		}
		controlPanel.disableButtons();

		currentColumn+=1;
		tired++;
		checkTurtle();
		
		jturtle.runRight();
		hMove=ANIM_HMOVE;
		vMove=0;
		count=HCOUNT;

		round_off_h+=ROUND_OFF_H;
		if (round_off_h>=ANIM_HMOVE) {
			count+=1;
			round_off_h-=ANIM_HMOVE;
		}

		animator.start();
	}

	public void moveDown() {
		if (count!=0) return;
		turtle.goSouth();
		if (tired==turtle.TIRED || currentRow==ROWS) {
			checkTurtle();
			return;
		}
		controlPanel.disableButtons();

		currentRow+=1;
		tired++;
		checkTurtle();

		hMove=0;
		vMove=ANIM_VMOVE;
		count=VCOUNT;

		round_off_v+=ROUND_OFF_V;
		if (round_off_v>=ANIM_VMOVE) {
			count+=1;
			round_off_v-=ANIM_VMOVE;
		}

		animator.start();
	}

	public void moveUp() {
		if (count!=0) return;
		turtle.goNorth();
		if (tired==turtle.TIRED || currentRow==1) {
			checkTurtle();
			return;
		}
		controlPanel.disableButtons();

		currentRow-=1;
		tired++;
		checkTurtle();
		
		hMove=0;
		vMove=-ANIM_VMOVE;
		count=VCOUNT;
		
		round_off_v-=ROUND_OFF_V;
		if (-round_off_v>=ANIM_VMOVE) {
			count+=1;
			round_off_v+=ANIM_VMOVE;
		}

		animator.start();
	}

	public void sleep() {
		if (count!=0) return;
		tired=0;
		turtle.sleep();
		checkTurtle();
	}
	
	public void home() {
		if (count!=0) return;
		turtle.goHome();
		currentRow=HOME_Y;
		currentColumn=HOME_X;
		tired=0;
		
		jturtle.setLocation(491,114);
		round_off_h=1.5;
		round_off_v=0.75;
		checkTurtle();
	}

	private void checkTurtle() {
		int tx=turtle.getLocation().getCol();
		int ty=turtle.getLocation().getRow();
		if (tx!=currentColumn || ty!=currentRow) {
			JOptionPane.showMessageDialog(this, "Turtle has wrong location (column="+
				tx+", row="+ty+") instead of (column="+currentColumn+", row="+currentRow+").",
				"Turtle",JOptionPane.ERROR_MESSAGE);
			System.exit(0);
		}
		if (tired!=turtle.getLevelOfTired()) {
			JOptionPane.showMessageDialog(this, "Turtle has wrong tiredness level!", "Turtle",
				JOptionPane.ERROR_MESSAGE);
			System.exit(0);
		}
	}
	
	
	public void actionPerformed(ActionEvent e) {
		if (count==0) {
			animator.stop();
			jturtle.stop();
			controlPanel.enableButtons();
			
			//Point d=jturtle.getLocation();
			//System.out.println("x: "+d.x+" y: "+d.y+" "+round_off_h+" "+round_off_v);
			return;
		}
		Point d=jturtle.getLocation();
		d.x += hMove;
		d.y += vMove;
		//System.out.println("x: "+d.x+" y: "+d.y);
		jturtle.setLocation(d);
		count--;
	}


	/**
	* @param args the command line arguments
	*/
	public static void main (String args[]) {
		new TurtleIsland().show();
	}

}

class GridPanel extends JComponent {

	private int rows,cols;

	GridPanel(int rows, int cols) {
		this.rows=rows;
		this.cols=cols;
		setOpaque(false);
	}

	public void paint(Graphics g) {
		g.setColor(Color.black);
		Dimension d=getSize();
		double h=d.height/(double)rows;
		for (int i=1; i<rows; i++) {
			int y=(int)(h*i)-1;
			g.drawLine(0,y,d.width,y);
		}

		double v=d.width/(double)cols;
		for (int i=1; i<cols; i++) {
			int x=(int)(v*i)-1;
			g.drawLine(x,0,x,d.height);
		}

	}
}


class Controls extends JPanel implements ActionListener {
	TurtleIsland target;

	TextField fileNameField;
	JButton leftBtn,rightBtn,downBtn,upBtn;
	JButton sleepBtn,homeBtn;
	JLabel location, tiredness;
	
	JButton buttons[]=new JButton[6];
	
	public Controls(TurtleIsland target) {
		this.target = target;
		//setLayout(new BorderLayout());
		setLayout(new FlowLayout(FlowLayout.CENTER,40,8));
		setBackground(Color.lightGray);

		JPanel p=new JPanel();
		p.setLayout(new BorderLayout());
		//add(p, BorderLayout.WEST);
		add(p);

		{
			rightBtn = new JButton("East");
			rightBtn.addActionListener(this);
			p.add(rightBtn,BorderLayout.EAST);
			leftBtn = new JButton("West");
			leftBtn.addActionListener(this);
			p.add(leftBtn,BorderLayout.WEST);

			downBtn = new JButton("South");
			downBtn.addActionListener(this);
			p.add(downBtn,BorderLayout.SOUTH);

			upBtn = new JButton("North");
			upBtn.addActionListener(this);
			p.add(upBtn,BorderLayout.NORTH);
		}

		p=new JPanel();
		p.setLayout(new BorderLayout());
		//add(p, BorderLayout.EAST);
		add(p);
		{
			sleepBtn = new JButton("Sleep");
			sleepBtn.addActionListener(this);
			p.add(sleepBtn,BorderLayout.SOUTH);

			homeBtn = new JButton("Home");
			homeBtn.addActionListener(this);
			p.add(homeBtn,BorderLayout.NORTH);
		}

		p=new JPanel();
		p.setLayout(new BorderLayout());
		//add(p, BorderLayout.CENTER);
		add(p);
		{
			location=new JLabel("location: ");
			p.add(location,BorderLayout.NORTH);
			tiredness=new JLabel("tiredness: ");
			p.add(tiredness,BorderLayout.SOUTH);
		}

		buttons[0]=leftBtn;	buttons[1]=rightBtn; buttons[2]=downBtn;
		buttons[3]=upBtn; buttons[4]=sleepBtn; buttons[5]=homeBtn;
	}

	public void actionPerformed(ActionEvent e) {
		//try	{
		if (e.getSource()==leftBtn)
			 target.moveLeft();
		else if (e.getSource()==rightBtn)
			 target.moveRight();
		else if (e.getSource()==downBtn)
			target.moveDown();
		else if (e.getSource()==upBtn)
			target.moveUp();
		else if (e.getSource()==sleepBtn)
			target.sleep();
		else if (e.getSource()==homeBtn)
			target.home();
		showLocation(target.currentRow,target.currentColumn);
		showTiredness(target.tired);
		//} catch (NoSuchMethodError ee)	{
		//	JOptionPane.showMessageDialog(null, ee.getMessage(), "Turtle",
		//		JOptionPane.ERROR_MESSAGE);
		//}

	}

	public void showLocation(int x, int y) {
		location.setText("location: "+x+","+y);
	}

	public void showTiredness(int t) {
		tiredness.setText("tiredness: "+t);
	}
	
	public void disableButtons() {
		for (int i=0; i<buttons.length; i++)
			buttons[i].setEnabled(false);
	}

	public void enableButtons() {
		for (int i=0; i<buttons.length; i++)
			buttons[i].setEnabled(true);
	}
}

class JTurtle extends JLabel {
	private static ImageIcon runLeft;
	private static ImageIcon runRight;
	private static ImageIcon stopRight;
	private static ImageIcon stopLeft;
	static {
		try {
			runLeft=new ImageIcon(ClassLoader.getSystemResource("pic/turtle_l.gif"));
			runRight=new ImageIcon(ClassLoader.getSystemResource("pic/turtle_r.gif"));
			stopRight=new ImageIcon(ClassLoader.getSystemResource("pic/turtle_stop_r.gif"));
			stopLeft=new ImageIcon(ClassLoader.getSystemResource("pic/turtle_stop_l.gif"));
		} catch(Exception e) {
			System.out.println("The 'pic' directory is not found!");
			System.exit(1);
		}
	}
	
	public static final int RUN_RIGHT=1;
	public static final int RUN_LEFT=2;
	public static final int STOP_RIGHT=3;
	public static final int STOP_LEFT=4;
	public static final int DOWN=5;
	public static final int UP=6;
	public int state=0;

	private boolean isRight=true;
	private boolean isDown=false;

	JTurtle(boolean isRight) {
		this.isRight=isRight;
		if (isRight)
			setIcon(stopRight);
		else
	   		setIcon(stopLeft);
	}

	void runLeft() {
		setIcon(runLeft);
		isRight=false;
	}

	void runRight() {
		setIcon(runRight);
		isRight=true;
	}
/*
	void turnDown() {
		if (state!=DOWN) {
			setIcon(runDown);
			state=DOWN;
		}
	}

	void turnUp() {
		if (state!=UP) {
			setIcon(runUp);
			state=UP;
		}
	}
*/
	void stop() {
		if (isRight) {
		   setIcon(stopRight);
		}
		else {
		   setIcon(stopLeft);
		}
	}
}
